#include "mainwidget.h"
#include "ui_mainwidget.h"
#include <QDebug>
#include <QDir>
#include <QJsonObject>
#include <QPluginLoader>

MainWidget::MainWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::MainWidget)
{
    ui->setupUi(this);
    loadPrinterPlugins();
////  Пример кода загрузки и использования плагина с известным именем
//    QPluginLoader loader;
//    loader.setFileName("printers/PrinterPlugin");
//    PrinterInterface* printer = qobject_cast<PrinterInterface*>(loader.instance());
//    if (printer) {
//        printer->printDocument(ui->edtText->toPlainText());
//    }
//    else {
//        qDebug() << loader.errorString();
//    }
//    PrinterInterface* printer2 = qobject_cast<PrinterInterface*>(loader.instance());
//    printer2->printDocument(ui->edtText->toPlainText());
//    loader.unload();
//    loader.setFileName("printers/PrinterPlugin2");
//    PrinterInterface* printer3 = qobject_cast<PrinterInterface*>(loader.instance());
//    printer3->printDocument(ui->edtText->toPlainText());
}

MainWidget::~MainWidget()
{
    delete ui;
}

void MainWidget::loadPrinterPlugins()
{
    QDir dir(qApp->applicationDirPath());
    dir.cd("printers");
    QStringList filter;
    filter << "*.dll" << "*.so";
    QPluginLoader loader;
    //Поиск всех файлов с расширением библиотеки в подпапке printers
    foreach(QFileInfo info, dir.entryInfoList(filter)) {
        loader.setFileName(info.absoluteFilePath());
        //Попытка создать экземпляр корневого класса плагина
        PrinterInterface* printer = qobject_cast<PrinterInterface*>(loader.instance());
        //Экземпляр успешно создан
        if (printer) {
            //Получение значения параметра PrinterName, определенного в файле мета-данных плагина
            QString name = loader.metaData().value("MetaData").toObject().value("PrinterName").toString();
            //Добавление информации о плагине в combobox
            ui->cmbPrinters->addItem(name);
            //Добавление указателя на экземпляр корневого класса плагина в список загруженных
            _printers.append(printer);
        }
        //Ошибка при загрузке плагина или при попытке привести экземпляр корневого класса к интерфейсу PrinterInterface
        else {
            //Вывод сообщения об ошибке в консоль
            qDebug() << loader.errorString();
        }
    }
}

void MainWidget::on_btnPrint_clicked()
{
    //Вызов метода печати выбранного в combobox-е плагина принтера
    _printers.at(ui->cmbPrinters->currentIndex())->printDocument(ui->edtText->toPlainText());
}















